use Image::Magick ;


sub run {
	my ( $type, $cwd ) = @_ ;
	
	
	my $actual_cwd = $fckperl_config{ 'UserFilesPath' } . "/" . $type . $cwd ;
	
	my @folders ;
	my @files ;
	
	if ( opendir( DIR, path( $actual_cwd ) ) ) {
		while ( defined( my $filename = readdir( DIR ) ) ) {
			next if ( $filename eq "." ) ;
			next if ( $filename eq ".." ) ;
			
			Jcode::convert( \$filename, 'utf8', $fckperl_config{ 'FileEncoding' } );
			
			if ( -d path( $actual_cwd . $filename ) ) {
				# check if$fckphp_configured not to show this folder
				
				my $hide = 0;
				
				for( my $i = 0; $i < @{ $fckperl_config{ 'ResourceAreas' }{ $type }{ 'HideFolders' } }; $i++ ) {
					my $pattern = $fckperl_config{ 'ResourceAreas' }{ $type }{ 'HideFolders' }[ $i ] ;
					
					if ( $filename =~ /$pattern/ ) {
						$hide = 1 ;
						last ;
					}
				}
				
				next if ( $hide ) ;
				
				push( @folders, $filename ) ;
			} else {
				$filename =~ /^.*\.([^\.]*)$/ ;
				
				next if ( ! in_array( lc($1) , \@{ $fckperl_config{ 'ResourceAreas' }{ $type }{ 'AllowedExtensions' } } ) ) ;
				
				# check if $fckphp_configured not to show this file
				
				my $hide = 0 ;
				
				for( my $i = 0; $i < @{ $fckperl_config{ 'ResourceAreas' }{ $type }{ 'HideFiles' } }; $i++ ) {
					my $pattern = $fckperl_config{ 'ResourceAreas' }{ $type }{ 'HideFiles' }[ $i ] ;
					
					if ( $filename =~ /$pattern/ ) {
						$hide = 1 ;
						last ;
					}
				}
				
				next if ( $hide ) ;
				
				push( @files, $filename ) ;
			}
		}
		
		closedir( DIR ); 
	}
	
	@folders = sort( @folders ) ;
	@files = sort( @files ) ;
	
	print 'Content-Type: application/xml; charset=utf-8' . "\n" ;
	print "\n" ;
	print '<?xml version="1.0" encoding="utf-8" ?>' . "\n" ;
	print '<Connector command="GetFoldersAndFiles" resourceType="' . XMLEncode( $type ) . '">' . "\n" ;
	print '    <CurrentFolder path="' . XMLEncode( $cwd ) . '" url="' . XMLEncode( url( $actual_cwd ) ) . '" />' . "\n" ;
	
	print "    <Folders>\n" ;
	for ( my $i = 0; $i < @folders; $i++ ) {
		print '        ' ;
		print '<Folder' ;
		print ' name="' . XMLEncode( $folders[ $i ] ) . '"' ;
		print ' url="' . XMLEncode( url( $folders[ $i ] ) ) . '"' ;
		print ' />' ;
		print "\n" ;
	}
	print "    </Folders>\n" ;
	
	print "    <Files>\n" ;
	for ( my $i = 0; $i < @files; $i++ ) {
		my $file = path( $actual_cwd . $files[ $i ] ) ;
		my $size = ceil( ( stat $file )[ 7 ] / 1024 ) ;
		my $width = '' ;
		my $height = '' ;
		my $imagetype = getType( $file ) ;
		
		if ( $imagetype ne '' ) {
			my $img = Image::Magick->new() ;
			$img->Read( $file ) ;
			( $width, $height ) = $img->Get( 'width', 'height' ) ;
			undef $img ;
		}
		
		print '        ' ;
		print '<File' ;
		print ' name="' . XMLEncode( $files[ $i ] ) . '"' ;
		print ' url="' . XMLEncode( url( $files[ $i ] ) ) . '"' ;
		print ' imagetype="' . $imagetype . '"' ;
		print ' size="' . $size . '"' ;
		print ' width="' . $width . '"' ;
		print ' height="' . $height . '"' ;
		print ' />' ;
		print "\n" ;
	}
	print "    </Files>\n" ;
	
	print "</Connector>\n" ;
}


sub getExtension {
	my ( $filename ) = @_ ;
	
	$filename =~ /^.*\.([^\.]*)$/ ;
	return $1 ;
}


sub getType {
	my ( $file ) = @_ ;
	
	
	open( my $fh, "<" . $file );
	
	read( $fh, my $start4, 4 ) ;
	my $start3 = substr( $start4, 0, 3 ) ;
	
	close( $fh ) ;
	
	return "png"	if ( $start4 eq "\x89PNG" ) ;
	return "gif"	if ( $start3 eq "GIF" ) ;
	return "jpeg"	if ( $start3 eq "\xFF\xD8\xFF" ) ;
	return "jpeg"	if ( $start4 eq "hsi1" ) ;
	
	my $ext = getExtension( $file ) ;
	
	return "gif"	if ( $ext eq "gif" ) ;
	return "jpeg"	if ( $ext eq "jpg" ) ;
	return "jpeg"	if ( $ext eq "jpeg" ) ;
	return "png"	if ( $ext eq "png" ) ;
	
	return '' ;
}


1;
