#!/usr/bin/perl

=comment
Perl Connector for the FCKEditor v2 File Manager
Written By Grant French, UK, Sept 2004
http://www.mcpuk.net

FCKEditor - By Frederico Caldeira Knabben
http://www.fckeditor.net

File Description:
Main connector file, implements the State Pattern to 
redirect requests to the appropriate class based on 
the command name passed.
=cut

use strict ;
use POSIX ;
use Jcode ;

use vars qw(%fckperl_config $DIRECTORY_SEPARATOR) ;
require( "config.pl" ) ;

use vars qw(%_GET) ;


#==============================================================================
# configure

#------------------------------------------------------------------------------
# HTTP over SSL Detection (shouldnt require changing

$fckperl_config{'prot'} = ((exists($ENV{'HTTPS'}) && ($ENV{'HTTPS'} =~ /^on$/i)) ? "https://" : "http://");

#------------------------------------------------------------------------------
# Prefix added to image path before sending back to editor

$fckperl_config{'urlprefix'} = $fckperl_config{'prot'} . $ENV{'SERVER_NAME'};

#------------------------------------------------------------------------------
# DIRECTORY_SEPARATOR

$DIRECTORY_SEPARATOR = (exists($ENV{'OS'}) ? (($ENV{'OS'} =~ /^Windows/) ? '\\' : '/') : '/');

#------------------------------------------------------------------------------
# Icon lookup directory

$fckperl_config{'iconLookupDir'} = dirname($ENV{'SCRIPT_NAME'}) . "images/";

#------------------------------------------------------------------------------
# The physical path to the document root

my $sRealPath = dirname($ENV{'SCRIPT_FILENAME'}) ;
Jcode::convert( \$sRealPath, 'utf8', $fckperl_config{'FileEncoding'} ) ;

my @sSelfPath_part = split( '/', dirname($ENV{'SCRIPT_NAME'}), -1 ) ;

for (my $n = 0; $n < @sSelfPath_part; $n++) {
    $sSelfPath_part[$n] = urldecode($sSelfPath_part[$n]) ;
    Jcode::convert( \$sSelfPath_part[$n], 'utf8', $fckperl_config{'FileEncoding'} );
}

my $sSelfPath = join( $DIRECTORY_SEPARATOR, @sSelfPath_part ) ;

$fckperl_config{'basedir'} = substr( $sRealPath, 0, length( $sRealPath ) - length( $sSelfPath ) ) ;

#==============================================================================

#==============================================================================
# GET

%_GET = query_decode( $ENV{ 'QUERY_STRING' } ) ;

#==============================================================================
# main process

outputHeaders();

# Get the passed data
my $command	= getParam( 'Command', "" ) ;
my $type	= getParam( 'Type', "File" ) ;
my $cwd		= getParam( 'CurrentFolder', "/" ) ;

$fckperl_config{ 'UserFilesPath' } = getParam( 'ServerPath', $fckperl_config{ 'UserFilesPath' } ) ;
$fckperl_config{ 'UserFilesPath' } =~ s/\/$// ;

if ( ! in_array( $command, \@{ $fckperl_config{ 'Commands' } } ) ) {
	# No reason for me to be here.
	print "Invalid command '{$command}'.";
	exit( 0 );
}

# bit of validation
if ( ! in_array( $type, \@{ $fckperl_config{ 'ResourceTypes' } } ) ) {
	print "Invalid resource type.";
	exit( 0 );
}

require( "Commands/${ command }.pl" ) ;
&run( $type, $cwd ) ;
exit( 0 );


#==============================================================================
# Functions


#-----------------------------------------------------------------------
# in array

sub in_array {
	my ( $key, $array_ref, $is_string ) = @_ ;
	$is_string = 1 if ( ! defined( $is_string ) ) ;
	
	my @array = @$array_ref ;
	
	if ( $is_string ) {
		foreach my $in ( @array ) {
			return( 1 ) if ( $in eq $key ) ;
		}
	} else {
		foreach my $in ( @array ) {
			return( 1 ) if ( $in == $key ) ;
		}
	}
	
	return( 0 ) ;
}


#-----------------------------------------------------------------------
# directory name

sub dirname {
	my ( $path ) = @_ ;
	$path =~ s/\/+[^\/]*$// ;
	return $path . '/' ;
}


#-----------------------------------------------------------------------
# base name

sub basename {
	my ( $path ) = @_ ;
	my @part = split( /[\\\/]/, $path, -1 ) ;
	return pop( @part ) ;
}

#-----------------------------------------------------------------------
# string urlencode($string)

sub urlencode($) {
	my $string = shift @_ ;
	
	$string =~ s/([^-.0-9A-Z_a-z ])/sprintf("%%%02X",ord($1))/eg ;
	$string =~ tr/ /+/ ;
	
	return( $string ) ;
}

#-----------------------------------------------------------------------
# string urldecode($string)

sub urldecode($) {
	my $string = shift @_ ;
	
	$string =~ tr/+/ / ;
	$string =~ s/%([A-Fa-f0-9][A-Fa-f0-9])/pack("c", hex($1))/ge ;
	
	return( $string ) ;
}

#------------------------------------------------------------------------------
# decode query string

sub query_decode {
	my ( $string, $delimiter ) = shift @_ ;
	$delimiter = '&' if ( ! defined( $delimiter ) ) ;
	
	
	my %query = () ;
	my ( $key, $val ) ;
	
	return( %query ) unless( $string ) ;
	
	foreach my $part ( split( /$delimiter/, $string, -1 ) ) {
		( $key, $val ) = split( /=/, $part, -1 ) ;
		$key = &urldecode( $key ) ;
		$val = &urldecode( $val ) ;
		$val =~ s/\r\n/\n/g ;
		$query{ $key } = $val ;
	}
	
	return( %query ) ;
}


#------------------------------------------------------------------------------
# get parametor with default

sub getParam {
	my ($name, $default) = @_ ;
	
	
	return $default if ( ! defined( $_GET{ $name } ) ) ;
	return $default if ( $_GET{ $name } eq "" ) ;
	
	return $_GET{ $name } ;
}

#------------------------------------------------------------------------------
# string gmdate($time)

sub gmdate {
	my ( $time ) = shift @_;
	my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = defined($time) ? gmtime($time) : gmtime() ;
	
	my $name_wday = ( "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" )[ $wday ] ;
	my $name_mon = ( "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" )[ $mon ] ;
	my $name_year = $year + 1900 ;
	
	return( sprintf( "%s, %d %s %d %02d:%02d:%02d GMT", $name_wday, $mday, $name_mon, $name_year, $hour, $min, $sec ) ) ;
}


#------------------------------------------------------------------------------
# output Headers

sub outputHeaders {

	# Anti browser caching headers
	# Borrowed from fatboy's implementation  (fatFCK@code247.com)
	
	# ensure file is never cached
	# Date in the past
	print "Expires: Mon, 26 Jul 1997 05:00:00 GMT\n" ;
	
	# always modified
	print "Last-Modified: " . gmdate() . "\n" ;
	
	# HTTP/1.1
	print "Cache-Control: no-store, no-cache, must-revalidate\n" ;
	print "Cache-Control: post-check=0, pre-check=0\n" ;
	
	# HTTP/1.0
	print "Pragma: no-cache\n" ;
}


sub path {
	my ($path) = @_ ;
	
	
	$path =~ s/\//$DIRECTORY_SEPARATOR/g ;
	$path = Jcode::convert( \$path, $fckperl_config{ 'FileEncoding' }, 'utf8' ) ;
	
	return $fckperl_config{ 'basedir' } . $path ;
}


sub mapFolder {
	my ( $dir ) = @_ ;
	
	
	my $parent = $dir ;
	$parent =~ s/[^\/]+\/*$// ;
	
	if ( ! -e path( $parent ) ) {
		return 0 if ( mapFolder( $parent ) == 0 ) ;
	}
	
	return 1 if ( -e path( $dir ) ) ;
	
	mkdir( path( $dir ), 0755 ) ;
	
	return ( -d path( $dir ) ) ? 1 : 0 ;
}


sub url {
	my ( $url ) = @_ ;
	
	
	my @url_part = split( /\//, $url, -1 ) ;
	
	for ( my $n = 0; $n < @url_part; $n++ ) {
		my $part = $url_part[ $n ] ;
		Jcode::convert( \$part, $fckperl_config{ 'FileEncoding' }, 'utf8' ) ;
		$url_part[ $n ] = urlencode( $part ) ;
	}
	
	return join( '/', @url_part ) ;
}


sub XMLEncode {
	my ($str) = @_ ;
	
	
	$str =~ s/&/&amp;/g ;
	$str =~ s/</&lt;/g ;
	$str =~ s/>/&gt;/g ;
	$str =~ s/"/&quot;/g ;
	$str =~ s/'/&apos;/g ;
	
	return $str;
}
